"""
    Dummy conftest.py for pysaleryd.
"""
import asyncio

import aiohttp
import pytest
import pytest_asyncio
from aiohttp import web

@pytest.fixture(scope="session")
def event_loop():
    """Overrides pytest default function scoped event loop"""
    policy = asyncio.get_event_loop_policy()
    loop = policy.new_event_loop()
    yield loop
    loop.close()

@pytest_asyncio.fixture(scope="session", autouse=True)
async def server():
    """Websocket test server"""
    async def websocket_handler(request):

        ws = web.WebSocketResponse()
        await ws.prepare(request)

        async for msg in ws:
            if msg.type == aiohttp.WSMsgType.TEXT:
                while True:
                    await ws.send_str("#MF: 1+ 0+ 2+\r")
                    await asyncio.sleep(0.5)
        return ws

    app = web.Application()
    app.add_routes([web.get('/', websocket_handler)])
    runner = web.AppRunner(app)
    await runner.setup()
    site = web.TCPSite(runner, 'localhost', 3001)
    await site.start()
    return site








from pysaleryd.skeleton import main

__author__ = "Björn Dalfors"
__copyright__ = "Björn Dalfors"
__license__ = "MIT"


def test_main(capsys):
    """CLI Tests"""
    # capsys is a pytest fixture that allows asserts against stdout/stderr
    # https://docs.pytest.org/en/stable/capture.html
    main(["--host", "192.168.1.151", "--port", "3001", "--listen", "-t", "3"])
    captured = capsys.readouterr()
    assert not captured.err

    main(["--host", "192.168.1.151", "--port", "3001", "--send", "--key", "MF", "--data", "0"])
    captured = capsys.readouterr()
    assert not captured.err



import logging
import pytest

from pysaleryd.utils import Parser, ParseError

__author__ = "Björn Dalfors"
__copyright__ = "Björn Dalfors"
__license__ = "MIT"


_LOGGER = logging.getLogger(__name__)

@pytest.fixture
def parser() -> Parser:
    return Parser()

def test_parse_list_from_str(parser: Parser):
    (key, value) = parser.from_str("#MF: 1+ 0+ 2\r")
    assert key == "MF"
    assert isinstance(value, list)
    assert value[0] == 1
    assert value[1] == 0
    assert value[2] == 2

def test_parse_int_from_list_str(parser: Parser):
    (key, value) = parser.from_str("#MF: 1+ 0+ 2+ 30\r")
    assert key == "MF"
    assert isinstance(value, list)
    assert value[0] == 1
    assert value[1] == 0
    assert value[2] == 2
    assert value[3] == 30

def test_parse_int_from_str(parser: Parser):
    (key, value) = parser.from_str("#*XX:0\r")
    assert key == "*XX"
    assert isinstance(value, int)
    assert value == 0

def test_parse_str_from_str(parser: Parser):
    (key, value) = parser.from_str("#*XX: xxx\r")
    assert key == "*XX"
    assert isinstance(value, str)
    assert value == "xxx"

    (key, value) = parser.from_str("#*XX:    1.x.1\r")
    assert key == "*XX"
    assert isinstance(value, str)
    assert value == "1.x.1"

def test_parse_error(parser: Parser):
    did_throw = False
    try:
        parser.from_str("wer")
    except ParseError:
        did_throw = True
    assert did_throw



"""HRV System client"""

import asyncio
import logging
from typing import Callable
import aiohttp

from .websocket import WSClient, Signal, State
from .utils import ParseError, Parser

_LOGGER: logging.Logger = logging.getLogger(__name__)

class Client:
    """Client to manage communication with HRV"""

    def __init__(self, url: str, port: int, session: aiohttp.ClientSession):
        self._url = url
        self._port = port
        self._session = session
        self._data = {}
        self._handlers = []
        self._socket = WSClient(self._session, self._url, self._port, self._handler)
        self._parser = Parser()

    async def __aenter__(self):
        """Start socket and wait for connection"""
        await self.connect()
        return self

    async def __aexit__(self, type, value, traceback):
        self.disconnect()

    async def connect(self):
        """Connect to system and wait for connection"""
        async def check_connection():
            while self._socket.state != State.RUNNING:
                await asyncio.sleep(0.2)

        self._socket.start()
        await asyncio.wait_for(check_connection(), 10)

    def disconnect(self):
        """Disconnect from system"""
        self._socket.stop()

    def add_handler(self, handler: Callable[[str], None]):
        """Add event handler"""
        self._handlers.append(handler)

    async def _handler(self, signal: Signal, data: str, state: State = None):
        """Call handlers if data"""
        if signal == Signal.DATA:
            try:
                (key, value) = self._parser.from_str(data)
                self._data[key] = value
            except ParseError:
                pass

            for handler in self._handlers:
                try:
                    handler(data)
                except Exception as exc:
                    _LOGGER.warning("Failed to call handler", exc_info=True)

    @property
    def state(self):
        """Get socket internal socket state"""
        return self._socket.state

    @property
    def data(self):
        """Get data from system"""
        return self._data

    async def send_command(self, key, value: str | int):
        """Send command to HRV"""
        message = self._parser.to_str(key, value)
        async def ack_command():
            """Should probably ack command here, just sleep for now"""
            await asyncio.sleep(2)

        await self._socket.send_message(message)
        await asyncio.gather(ack_command())



import asyncio
import aiohttp
import logging
import pytest
import pytest_asyncio

from pysaleryd.client import Client, State

__author__ = "Björn Dalfors"
__copyright__ = "Björn Dalfors"
__license__ = "MIT"

@pytest_asyncio.fixture
async def hrv_client():
    """HRV Client"""
    async with aiohttp.ClientSession() as session:
        async with Client("0.0.0.0", 3001, session) as client:
            yield client

@pytest.mark.asyncio
async def test_client_connect(hrv_client: Client):
    """client tests"""
    assert hrv_client.state == State.RUNNING

@pytest.mark.asyncio
async def test_handler(hrv_client: Client, mocker):
    """Test handler callback"""
    handler = mocker.Mock()
    def broken_handler(data):
        raise Exception()
    
    hrv_client.add_handler(broken_handler)
    hrv_client.add_handler(handler)
    await asyncio.sleep(3)
    handler.assert_called()

@pytest.mark.asyncio
async def test_get_data(hrv_client: Client, mocker):
    """Test get data"""
    await asyncio.sleep(1)
    assert isinstance(hrv_client.data, dict)

@pytest.mark.asyncio
async def test_reconnect(hrv_client: Client, mocker):
    """Test reconnect"""
    await hrv_client._socket._ws.close()
    await asyncio.sleep(1)
    assert hrv_client.state == State.RUNNING

@pytest.mark.asyncio
async def test_send_command(hrv_client: Client, mocker):
    """Test send command"""
    await hrv_client.send_command("MF", "0")

@pytest.mark.asyncio
async def test_disconnect(hrv_client: Client, mocker):
    """Test send command"""
    hrv_client.disconnect()
    await asyncio.sleep(2)
    assert hrv_client.state == State.STOPPED
    await asyncio.sleep(2)
    assert hrv_client._socket._ws.closed


 
import logging

_LOGGER: logging.Logger = logging.getLogger(__package__)


class ParseError(BaseException):
    pass

class Parser():
    def to_str(self, key, value):
        return (f"#{key}:{value}\r")

    def from_str(self, msg: str):
        """parse message"""
        try:
            if msg[0] == "#":
                if msg[1] == "$":
                    # ack message, strip ack char and treat as state update
                    msg = msg[1::]
                value = msg[1::].split(":")[1].strip()
                value = int(value) if value.isnumeric() else value

                if msg[1] != "*":
                    # messages not beginning with * are arrays of integers
                    # [value, min, max] or [value, min, max, time_left]
                    value = [
                        v if isinstance(v, int) else int(v.strip())
                        for v in value.split("+")
                    ]
                key = msg[1::].split(":")[0]
                parsed = (key, value)
                return parsed
        except Exception as exc:
            raise ParseError(f"Failed to parse message {msg}") from exc

        raise ParseError("Failed to parse message {msg}")
